@@ -8,12 +8,14 @@ import sys
 import time
 from json import JSONDecodeError
 
+import jsonpickle
 import requests
 import yaml
 from addict import Dict
 from appdirs import AppDirs
 from rich import print
 
+from kodexa.assistant import Assistant
 from kodexa.connectors import get_source
 from kodexa.connectors.connectors import get_caller_dir, FolderConnector
 from kodexa.model import Document, DocumentStore
@@ -119,9 +121,13 @@ OBJECT_TYPES = {
         "plural": "extension packs"
     },
     "pipelines": {
-        "name": "pipelines",
+        "name": "pipeline",
         "plural": "pipelines"
     },
+    "assistants": {
+        "name": "assistant",
+        "plural": "assistants"
+    },
     "actions": {
         "name": "action",
         "plural": "actions"
@@ -245,7 +251,7 @@ class KodexaPlatform:
 
     @staticmethod
     def deploy(ref: str, kodexa_object, name: str = None, description: str = None,
-               options=None, public=False, force_replace=False):
+               options=None, public=False, force_replace=False, dry_run=False, print_yaml=False):
 
         if '/' not in ref:
             logger.error("A ref must be valid, i.e. org_slug/pipeline_slug:version")
@@ -266,6 +272,8 @@ class KodexaPlatform:
 
         object_url = None
 
+        from kodexa import RemoteTableDataStore
+
         if isinstance(kodexa_object, Pipeline):
 
             metadata_object.name = 'New Pipeline' if metadata_object.name is None else metadata_object.name
@@ -307,76 +315,115 @@ class KodexaPlatform:
 
             metadata_object.type = 'store'
             metadata_object.storeType = 'MODEL'
+        elif isinstance(kodexa_object, RemoteTableDataStore):
+            object_url = 'stores'
+            metadata_object.name = 'New Store' if metadata_object.name is None else metadata_object.name
+            metadata_object.description = 'A table data store' if metadata_object.description is None else metadata_object.description
+
+            metadata_object.type = 'store'
+            metadata_object.storeType = 'TABLE'
         elif isinstance(kodexa_object, Workflow):
             metadata_object.name = 'New Workflow' if metadata_object.name is None else metadata_object.name
             metadata_object.description = 'A new workflow' if metadata_object.description is None else metadata_object.description
             object_url = 'workflows'
             metadata_object.type = 'workflow'
-            metadata_object.pipelines = list(map(lambda x: x.__dict__, kodexa_object.pipelines))
-            metadata_object.stores = list(map(lambda x: x.__dict__, kodexa_object.stores))
-            metadata_object.connectors = list(map(lambda x: x.__dict__, kodexa_object.connectors))
-            metadata_object.schedules = list(map(lambda x: x.__dict__, kodexa_object.schedules))
+            metadata_object.pipelines = jsonpickle.decode(jsonpickle.encode(kodexa_object.pipelines, unpicklable=False))
+            metadata_object.stores = jsonpickle.decode(jsonpickle.encode(kodexa_object.stores, unpicklable=False))
+            metadata_object.connectors = jsonpickle.decode(
+                jsonpickle.encode(kodexa_object.connectors, unpicklable=False))
+            metadata_object.schedules = jsonpickle.decode(jsonpickle.encode(kodexa_object.schedules, unpicklable=False))
             metadata_object.accessToken = kodexa_object.access_token
         elif isinstance(kodexa_object, Taxonomy):
             metadata_object.name = 'New Taxonomy' if metadata_object.name is None else metadata_object.name
             metadata_object.description = 'A new taxonomy' if metadata_object.description is None else metadata_object.description
             object_url = 'taxonomies'
             metadata_object.type = 'taxonomy'
-            metadata_object.taxons = list(map(lambda x: x.__dict__, kodexa_object.taxons))
+            metadata_object.enabled = kodexa_object.enabled
+            metadata_object.taxonomyType = kodexa_object.taxonomy_type
+            metadata_object.taxons = jsonpickle.decode(jsonpickle.encode(kodexa_object.taxons, unpicklable=False))
+        elif isinstance(kodexa_object, Assistant):
+            metadata_object.name = 'New Assistant Definition' if metadata_object.name is None else metadata_object.name
+            metadata_object.description = 'A new assistant definition' if metadata_object.description is None else metadata_object.description
+            object_url = 'assistants'
+            metadata_object.type = 'assistant'
+            metadata_object.fullDescription = kodexa_object.full_description
+            metadata_object.workflows = jsonpickle.decode(jsonpickle.encode(kodexa_object.workflows, unpicklable=False))
+            metadata_object.requiredStores = jsonpickle.decode(
+                jsonpickle.encode(kodexa_object.required_stores, unpicklable=False))
+            metadata_object.processingTaxonomies = jsonpickle.decode(
+                jsonpickle.encode(kodexa_object.processing_taxonomies, unpicklable=False))
+
+            # For our services we are actually going to dry run each of these
+            # as deploys to create the correct objects
+
+            metadata_object.services = []
+            for service in kodexa_object.services:
+                metadata_object.services.append(KodexaPlatform.deploy(f'./{service.slug}', service, dry_run=True))
+
         else:
             raise Exception("Unknown object type, unable to deploy")
 
-        logger.info(f"Deploying {metadata_object.type} {ref}")
+        if dry_run:
 
-        access_token = KodexaPlatform.get_access_token_details()
+            if print_yaml:
+                logger.info(f"Dry run output for {metadata_object.type} {ref}")
+                logger.info(f"YAML output:\n")
+                logger.info(yaml.dump(metadata_object.to_dict()))
 
-        if organization_slug not in access_token.organizationSlugs:
-            logger.error(f"You do not have access to the organization {organization_slug}")
-            raise Exception("Unable to deploy, no access to organization")
-
-        if object_version:
-            url = f"{KodexaPlatform.get_url()}/api/{object_url}/{organization_slug}/{object_slug}/{object_version}"
         else:
-            url = f"{KodexaPlatform.get_url()}/api/{object_url}/{organization_slug}/{object_slug}"
+            logger.info(f"Deploying {metadata_object.type} {ref}")
 
-        response = requests.get(url,
-                                headers={"x-access-token": KodexaPlatform.get_access_token(),
-                                         "content-type": "application/json"})
+            access_token = KodexaPlatform.get_access_token_details()
 
-        if response.status_code == 401:
-            logger.error(f"You do not have the permissions to access this {metadata_object.type}")
-            raise Exception("Not authorized on pipeline")
+            if organization_slug not in access_token.organizationSlugs:
+                logger.error(f"You do not have access to the organization {organization_slug}")
+                raise Exception("Unable to deploy, no access to organization")
 
-        if response.status_code == 404:
-            logger.info("Object doesn't exist, will deploy")
-            response = requests.post(f"{KodexaPlatform.get_url()}/api/{object_url}/{organization_slug}",
-                                     json=metadata_object.to_dict(),
-                                     headers={"x-access-token": KodexaPlatform.get_access_token(),
-                                              "content-type": "application/json"})
-            if response.status_code == 200:
-                logger.info("Deployed")
+            if object_version:
+                url = f"{KodexaPlatform.get_url()}/api/{object_url}/{organization_slug}/{object_slug}/{object_version}"
             else:
-                logger.error(response.text)
-                raise Exception("Unable to deploy")
-        elif response.status_code == 200:
-            logger.info(f"{ref} already exists")
-            if force_replace:
-                logger.info(f"Replacing {ref}")
-                response = requests.put(url,
-                                        json=metadata_object.to_dict(),
-                                        headers={"x-access-token": KodexaPlatform.get_access_token(),
-                                                 "content-type": "application/json"})
+                url = f"{KodexaPlatform.get_url()}/api/{object_url}/{organization_slug}/{object_slug}"
+
+            response = requests.get(url,
+                                    headers={"x-access-token": KodexaPlatform.get_access_token(),
+                                             "content-type": "application/json"})
+
+            if response.status_code == 401:
+                logger.error(f"You do not have the permissions to access this {metadata_object.type}")
+                raise Exception("Not authorized on pipeline")
+
+            if response.status_code == 404:
+                logger.info("Object doesn't exist, will deploy")
+                response = requests.post(f"{KodexaPlatform.get_url()}/api/{object_url}/{organization_slug}",
+                                         json=metadata_object.to_dict(),
+                                         headers={"x-access-token": KodexaPlatform.get_access_token(),
+                                                  "content-type": "application/json"})
                 if response.status_code == 200:
                     logger.info("Deployed")
                 else:
                     logger.error(response.text)
-                    raise Exception("Unable to deploy and replace")
+                    raise Exception("Unable to deploy")
+            elif response.status_code == 200:
+                logger.info(f"{ref} already exists")
+                if force_replace:
+                    logger.info(f"Replacing {ref}")
+                    response = requests.put(url,
+                                            json=metadata_object.to_dict(),
+                                            headers={"x-access-token": KodexaPlatform.get_access_token(),
+                                                     "content-type": "application/json"})
+                    if response.status_code == 200:
+                        logger.info("Deployed")
+                    else:
+                        logger.error(response.text)
+                        raise Exception("Unable to deploy and replace")
+                else:
+                    logger.warning("Not updating")
+                    return
             else:
-                logger.warning("Not updating")
-                return
-        else:
-            logger.error("Unable to deploy")
-            logger.error(response.content)
+                logger.error("Unable to deploy")
+                logger.error(response.content)
+
+        return metadata_object
 
     @staticmethod
     def list_objects(organization_slug, object_type):
@@ -504,7 +551,23 @@ class KodexaPlatform:
             return r.json()
         else:
             logger.error(r.text)
-            raise Exception("Unable to get action metadata, check your reference and platform settings")
+            raise Exception("Unable to get server information, check your platform settings")
+
+    @classmethod
+    def reindex(cls, object_type, ref):
+        object_type, object_type_metadata = resolve_object_type(object_type)
+        print(f"Reindexing {object_type_metadata['name']} [bold]{ref}[/bold]")
+        url_ref = ref.replace(':', '/')
+        r = requests.post(f"{KodexaPlatform.get_url()}/api/{object_type}/{url_ref}/_reindex",
+                          headers={"x-access-token": KodexaPlatform.get_access_token(),
+                                   "content-type": "application/json"})
+        if r.status_code == 401:
+            raise Exception("Your access token was not authorized")
+        elif r.status_code == 200:
+            return r.text
+        else:
+            logger.error(r.text)
+            raise Exception("Unable to reindex check your reference and platform settings")
 
 
 class RemoteSession:
@@ -519,7 +582,7 @@ class RemoteSession:
 
     def get_action_metadata(self, ref):
         logger.debug(f"Downloading metadata for action {ref}")
-        r = requests.get(f"{KodexaPlatform.get_url()}/api/actions/{ref}",
+        r = requests.get(f"{KodexaPlatform.get_url()}/api/actions/{ref.replace(':', '/')}",
                          headers={"x-access-token": KodexaPlatform.get_access_token()})
         if r.status_code == 401:
             raise Exception("Your access token was not authorized")
