@@ -55,7 +55,7 @@ class RemoteStore:
         from kodexa import KodexaPlatform
         import requests
         resp = requests.delete(
-            f"{KodexaPlatform.get_url()}/api/stores/{self.get_ref().replace(':','/')}/contents",
+            f"{KodexaPlatform.get_url()}/api/stores/{self.get_ref().replace(':', '/')}/contents",
             headers={"x-access-token": KodexaPlatform.get_access_token()})
 
         if resp.status_code == 200:
@@ -119,6 +119,7 @@ class ContentNode(object):
         <kodexa.model.model.ContentNode object at 0x7f80605e53c8>
         >>> current_content_node.add_child(new_page)
     """
+
     def __init__(self, document, node_type: str, content: Optional[str] = None,
                  content_parts: Optional[List[Any]] = None):
         if content_parts is None:
@@ -195,9 +196,19 @@ class ContentNode(object):
             new_content_node.content_parts = content_node_dict['content_parts']
 
         for dict_feature in content_node_dict['features']:
-            new_feature = new_content_node.add_feature(dict_feature['name'].split(':')[0],
-                                                       dict_feature['name'].split(':')[1],
-                                                       dict_feature['value'], dict_feature['single'], True)
+
+            feature_type = dict_feature['name'].split(':')[0]
+            if feature_type == 'tag':
+                new_content_node.add_feature(feature_type,
+                                             dict_feature['name'].split(':')[1],
+                                             dict_feature['value'], dict_feature['single'], True)
+            else:
+
+                # TODO we should convert to Tag?
+                new_content_node.add_feature(feature_type,
+                                             dict_feature['name'].split(':')[1],
+                                             dict_feature['value'], dict_feature['single'], True)
+
         for dict_child in content_node_dict['children']:
             new_content_node.add_child(ContentNode.from_dict(document, dict_child), dict_child['index'])
         return new_content_node
@@ -545,7 +556,7 @@ class ContentNode(object):
 
         if replace:
             for child in self.children:
-                self.parent = None
+                child.parent = None
             self.children = []
 
         for child in children:
@@ -574,7 +585,7 @@ class ContentNode(object):
         :param str new_tag_name: The name of the new tag.  This must be different from the existing_tag_name.
         """
         if existing_tag_name == None or new_tag_name == None or existing_tag_name == new_tag_name:
-            return # do nothing, just exit function
+            return  # do nothing, just exit function
 
         for node in self.select(selector):
             existing_tag_values = node.get_feature_value('tag', existing_tag_name)
@@ -591,13 +602,15 @@ class ContentNode(object):
                             new_uuid = str(uuid.uuid4())
 
                         previous_uuid = val['uuid']
-                        tag = Tag(start=val['start'], end=val['end'], value=val['value'], uuid=new_uuid, data=val['data'])
+                        tag = Tag(start=val['start'], end=val['end'], value=val['value'], uuid=new_uuid,
+                                  data=val['data'])
                         node.add_feature('tag', new_tag_name, tag)
                 else:
-                    tag = Tag(start=existing_tag_values['start'], end=existing_tag_values['end'], value=existing_tag_values['value'], uuid=str(uuid.uuid4()), data=existing_tag_values['data'])
+                    tag = Tag(start=existing_tag_values['start'], end=existing_tag_values['end'],
+                              value=existing_tag_values['value'], uuid=str(uuid.uuid4()),
+                              data=existing_tag_values['data'])
                     node.add_feature('tag', new_tag_name, tag)
 
-
     def collect_nodes_to(self, end_node):
         """
         Get the the sibling nodes between the current node and the end_node.
@@ -716,7 +729,6 @@ class ContentNode(object):
             else:
                 return str(uuid.uuid4())
 
-
         def tag_node_position(node_to_check, start, end, node_data, tag_uuid):
 
             content_length = 0
@@ -786,7 +798,6 @@ class ContentNode(object):
                                 end_offset = match.span()[1]
                                 tag_node_position(node, start_offset, end_offset, data, get_tag_uuid(tag_uuid))
 
-
     def get_tags(self):
         """
         Returns a list of the names of the tags on the given node
@@ -1313,6 +1324,8 @@ class Document(object):
         self.add_mixin('core')
         self.source: SourceMetadata = source
         self.labels: List[str] = []
+        self.taxonomies: List[str] = []
+        self.classifications: List[str] = []
 
         # Make sure we apply all the mixins
         registry.apply_to_document(self)
@@ -1439,6 +1452,8 @@ class Document(object):
                 'content_node': self.content_node.to_dict() if self.content_node else None,
                 'source': clean_none_values(dataclasses.asdict(self.source)),
                 'mixins': self._mixins,
+                'taxonomies': self.taxonomies,
+                'classifications': self.classifications,
                 'exceptions': self.exceptions,
                 'log': self.log,
                 'labels': self.labels,
@@ -1471,6 +1486,10 @@ class Document(object):
             new_document.source = SourceMetadata.from_dict(doc_dict['source'])
         if 'labels' in doc_dict and doc_dict['labels']:
             new_document.labels = doc_dict['labels']
+        if 'classifications' in doc_dict and doc_dict['labels']:
+            new_document.labels = doc_dict['labels']
+        if 'taxomomies' in doc_dict and doc_dict['taxomomies']:
+            new_document.labels = doc_dict['taxomomies']
         return new_document
 
     @staticmethod
